vec3 fresnelSchlick(float cosTheta, vec3 F0) {
    return F0 + (vec3(1.0) - F0)*pow(1.0 - cosTheta, 5.0);
}

float distributionGGX(vec3 N, vec3 H, float roughness) {
    float a = roughness*roughness;
    float a2 = a*a;
    float NdotH = max(dot(N, H), 0.0);
    float NdotH2 = NdotH*NdotH;

    float num = a2;
    float denom = (NdotH2*(a2 - 1.0) + 1.0);
    denom = PI*denom*denom;

    return num/denom;
}

float geometrySchlickGGX(float NdotV, float roughness) {
    float a = roughness*roughness;
    float r = (a + 1.0);
    float k = (r*r) / 8.0;

    float num = NdotV;
    float denom = NdotV*(1.0 - k) + k;

    return num/denom;
}

float geometrySmith(vec3 N, vec3 V, vec3 L, float roughness) {
    float NdotV = max(dot(N, V), 0.0);
    float NdotL = max(dot(N, L), 0.0);
    float ggx2 = geometrySchlickGGX(NdotV, roughness);
    float ggx1 = geometrySchlickGGX(NdotL, roughness);

    return ggx1*ggx2;
}

vec3 physicalBrdf(mediump vec3 L, mediump vec3 normal, mediump vec3 view, mediump vec3 albedo, mediump float metallic, mediump float roughness, mediump vec3 f0) {
    vec3 halfVec = normalize(view + L);

    float NDF = distributionGGX(normal, halfVec, roughness);
    float G = geometrySmith(normal, view, L, roughness);
    vec3 f = fresnelSchlick(max(dot(halfVec, view), 0.0), f0);

    vec3 kD = (vec3(1.0) - f)*(1.0 - metallic);

    vec3 numerator = NDF*G*f;
    float NdotL = max(dot(normal, L), 0.0);
    float denominator = 4.0*max(dot(normal, view), 0.0)*NdotL;
    vec3 specular = numerator/max(denominator, 0.001);

    return (kD*albedo/PI + specular)*NdotL;
}
